library(caret)        # featurePlot
library(factoextra)
library(cluster)            ## visualize clustering output
library(dendextend)   # extended dendrograms

#unsupervised learning
# k-means clustering - Check for best k
k <- 2
i <- grep("Petal", names(iris))
x <- iris[,i]
km <- kmeans(x, k)
km
plot(x, col="#FFFFFF")
text(x,labels = iris$Species, col=km$cluster)
points(km$centers, col=c(1:k), pch=8, cex=1.6, lwd=1.5)

featurePlot(x=iris[,1:4], y=iris[,5], plot="ellipse")

# How to find the optimal k?
ks <- 1:5
collect_within_ss <- sapply(ks, function(k){
  cl <- kmeans(x, k)
  cl$tot.withinss
})
plot(ks, collect_within_ss, type="b",
     ylab="Total within squared distances", xlab="Tested value of k")
silhouette(km$cluster, dist(x)) |> fviz_silhouette()
fviz_nbclust(x, kmeans, method = "silhouette", k.max = 10)
#
km <- kmeans(x, 2)
plot(x, col=km$cluster)
points(km$centers, col=c(1:2), pch=8, cex=1.6, lwd=1.5)

# hierarchical clustering
d <- dist(iris[,1:4])
hcl <- hclust(d)
hcl
hcl_dend <- as.dendrogram(hcl)
labels_colors(hcl_dend) <- as.numeric(iris$Species)
par(mfrow = c(1, 1))
plot(hcl_dend)
abline(h=3, col="red")
# grab cluster assignments for a value of k
# use either k=X or h=X
cutree(hcl, k=2)
cutree(hcl, h=3)
km$cluster
table(km$cluster, cutree(hcl, k=2))
par(mfrow = c(1, 2))
plot(iris$Petal.Length, iris$Sepal.Length, col = km$cluster, main = "k-means")
plot(iris$Petal.Length, iris$Sepal.Length, col = cutree(hcl, h = 3), main = "Hierarchical clustering")
dev.off()
par(mfrow = c(1, 3))
plot(iris$Petal.Length, iris$Sepal.Length, col = cutree(hcl, k = 2), main = "Hierarchical clustering")
plot(iris$Petal.Length, iris$Sepal.Length, col = cutree(hcl, k = 3), main = "Hierarchical clustering")
plot(iris$Petal.Length, iris$Sepal.Length, col = cutree(hcl, h = 3), main = "Hierarchical clustering")
dev.off()

table(iris$Species, km$cluster)
table(iris$Species, cutree(hcl,h=4))

# scaling
par(mfrow = c(1, 2))
mtcars |> dist() |> hclust() |> plot()
mtcars |> scale() |> dist() |> hclust() |> plot()
dev.off()
